#include <iostream.h>
#include <Bitmap.h>
#include <Alert.h>
#include <Dragger.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include <Window.h>
#include <stdio.h>
#include "rrlms.h"
#include "errors.h"

#include "rrclientReplicantView.h"

extern const char *app_signature;


rrclientReplicantView::rrclientReplicantView(BRect frame, const char *name)
	:BView(frame, name, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS)
{

	icon = new BBitmap(Bounds(), RRSmallIconColorSpace,0 ,0 );
	icon->SetBits(RRSmallIconBits, 256, 0, RRSmallIconColorSpace);
	
	BRect rect(Bounds());
	rect.OffsetTo(B_ORIGIN);
	rect.top = rect.bottom - 7;
	rect.left = rect.right - 7;
	AddChild(new BDragger(rect, this, B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM));
	
	BuildMenu();
}

rrclientReplicantView::rrclientReplicantView(BMessage *message)
	: BView(message)
{
	icon = new BBitmap(Bounds(), RRSmallIconColorSpace,0 ,0 );
	icon->SetBits(RRSmallIconBits, 256, 0, RRSmallIconColorSpace);
	
	BuildMenu();
}

void
rrclientReplicantView::BuildMenu()
{
	rrclientMenu = new BPopUpMenu("rrclientMenu", false, false, B_ITEMS_IN_COLUMN);
	BMenuItem *About = new BMenuItem("About", new BMessage());
	BMenuItem *Options = new BMenuItem("Options", new BMessage());
	BMenuItem *Exit = new BMenuItem("Exit",new BMessage());
	
	rrclientMenu->AddItem(About);
	rrclientMenu->AddItem(Options);
	rrclientMenu->AddItem(Exit);
	rrclientMenu->AddSeparatorItem();
	
	BMenuItem *Login = new BMenuItem("Login", new BMessage());
	BMenuItem *Logoff = new BMenuItem("Logoff", new BMessage());
	
	rrclientMenu->AddItem(Login);
	rrclientMenu->AddItem(Logoff);
	
}

rrclientReplicantView::~rrclientReplicantView()
{
delete icon;
}

rrclientReplicantView *
rrclientReplicantView::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data, "rrclientReplicantView"))
		return NULL;
	return new rrclientReplicantView(data);
}

status_t 
rrclientReplicantView::Archive(BMessage *data, bool deep) const
{
	uint32 winresizingmode = ResizingMode();
	((BView*)this)->BView::SetResizingMode(B_FOLLOW_LEFT|B_FOLLOW_TOP);
	BView::Archive(data, deep);
	((BView*)this)->SetResizingMode(winresizingmode);

	data->AddString("add_on", app_signature);
	return B_NO_ERROR;
}

void 
rrclientReplicantView::Draw(BRect rect)
{
	BView::Draw(rect);
	DrawBitmap(icon, BPoint(0,0));
}


void
rrclientReplicantView::MouseDown(BPoint point)
{
	BMenuItem *selected; 
	ConvertToScreen(&point); 
    selected = rrclientMenu->Go(point); 
    
    if (selected != NULL)
			{
				if (strcmp(selected->Label(),"About") == 0)
					(new BAlert("About rrclient", "rrclient\n\n\t By:Don Wurster\n\t dew8597@rit.edu","OK"))->Go();
				if (strcmp(selected->Label(),"Options") == 0)
				{
					BRect windowRect(300, 300, 550, 450);
					optionsWindow = new rrclientOptionsWindow("rrclient Options", windowRect);
					optionsWindow->Show();
				}
				if (strcmp(selected->Label(),"Exit") == 0)
				{
  					(new BAlert("Exit", "Goodbye...","Later"))->Go();
					RemoveFromDeskBar("rrclient");
				}
					
				if (strcmp(selected->Label(),"Login") == 0)
				{
					//(new BAlert("Login", "Login Now...","OK"))->Go();
					 int result = rrlmsLogin();
					 char result_msg[100];
					 sprintf(result_msg, GetResultMsg( result ));
					 (new BAlert("Result", result_msg,"OK"))->Go();
				}
				if (strcmp(selected->Label(),"Logoff") == 0)
				{
					//(new BAlert("Logoff", "Logoff Now...","OK"))->Go();
					 int result = rrlmsLogout();
					 char result_msg[100];
					 sprintf(result_msg, GetResultMsg( result ));
					 (new BAlert("Result", result_msg,"OK"))->Go();
				}
			}

}

void
rrclientReplicantView::MessageReceived(BMessage *message)
{
	switch( message->what )
	{
		case 'RPLY':
			  if( message->FindMessenger("result", &messenger ) != B_OK )
			break;
		default:
			BView::MessageReceived( message );
	}
}

void
rrclientReplicantView::AttachedToWindow()
{
	GetDeskBarMessenger();
	SetViewColor(0,0,0); 
	BView::AttachedToWindow(); 
   
}

BMessenger
*rrclientReplicantView::GetDeskBarMessenger(void)
{
	BMessage	request(B_GET_PROPERTY);
	BMessenger	to;
	BMessenger	result;


	request.AddSpecifier("Messenger");
	request.AddSpecifier("Shelf");

	request.AddSpecifier("View", "Status");
	request.AddSpecifier("Window", "Deskbar");
	to = BMessenger("application/x-vnd.Be-TSKB", -1);

	BMessage	reply;
	BMessenger	replyto( this );
	
	to.SendMessage(&request, this );

	return NULL;
}

void
rrclientReplicantView::RemoveFromDeskBar( const char *name )
{
	BMessage	request(B_DELETE_PROPERTY);		// Delete
	BMessage	uid_specifier(B_NAME_SPECIFIER);	// specifying via ID

	// IDs are specified using code like the following 3 lines:
	uid_specifier.AddString("name", name );
	uid_specifier.AddString("property", "Replicant");
	request.AddSpecifier(&uid_specifier);
	
	if( messenger.IsValid() )
		messenger.SendMessage(&request);
}

const char*
rrclientReplicantView::GetResultMsg( int result )
{
	const char* message;
	switch(result)
		{
			case SOCKET_ERROR:
				message = SOCKET_ERROR_MSG;
				break;
			case SERVER_CONNECT_ERROR:
				message = SERVER_CONNECT_ERROR_MSG;
				break;
			case SERVER_RECIEVE_ERROR:
				message = SERVER_RECIEVE_ERROR_MSG;
				break;
			case UNEXPECTED_PACKET_ERROR:
				message = UNEXPECTED_PACKET_ERROR_MSG;
				break;
			case SUCCESSFUL_LOGOUT:
				message = SUCCESSFUL_LOGOUT_MSG;
				break;
			case UNSUCCESSFUL_LOGOUT:
				message = UNSUCCESSFUL_LOGOUT_MSG;
				break;
			case SETTINGS_FILE_ERROR:
				message = SETTINGS_FILE_ERROR_MSG;
				break;
			case UNEXPECTED_RESPONSE:
				message = UNEXPECTED_RESPONSE_MSG;
				break;
			case UNSUCCESSFUL_LOGIN:
				message = UNSUCCESSFUL_LOGIN_MSG;
				break;
			case SUCCESSFUL_LOGIN:
				message = SUCCESSFUL_LOGIN_MSG;
				break;
			default:
				message = "Unknown Error!\n";
		}
	return message;
}
